package com.hzya.frame.plugin.lets.util;

import cn.hutool.core.lang.Assert;
import com.hzya.frame.beanutil.BeanUtil;
import com.hzya.frame.ttxofs.dto.ofssalesordersearch.DetailsDto;
import com.hzya.frame.ttxofs.dto.ofssalesordersearch.HeaderDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * 处理OFS销售订单，优惠金额分摊逻辑
 *
 * @Author：liuyang
 * @Package：com.hzya.frame.plugin.lets.util
 * @Project：kangarooDataCenterV3
 * @name：AmountAllocationUtil
 * @Date：2024/9/21 18:19
 * @Filename：AmountAllocationUtil
 */
@Component
public class OfsOrderSaleAmountAllocationUtil {

    Logger logger = LoggerFactory.getLogger(OfsOrderSaleAmountAllocationUtil.class);

    /**
     * TOC销售金额分摊
     *
     * @param headerDetailsDto OFS销售订单对象
     * @author liuyang
     */
    public void tocSalesAmountAllocation(com.hzya.frame.ttxofs.dto.ofssalesordersearch.HeaderDetailsDto headerDetailsDto) throws Exception {
        logger.info("==TOC销售金额分摊开始==");

        Assert.notNull(headerDetailsDto, "OFS销售订单对象不能为空");
        HeaderDto header = headerDetailsDto.getHeader();
        List<DetailsDto> detailsOld = headerDetailsDto.getDetails();

        //拷贝一份List集合，并移除没有实付金额的原始
        List<DetailsDto> details = copyDetailsDto(detailsOld);
        details.removeIf(dto -> dto.getTotalPayAmount() == null || !"0".equals(new BigDecimal(dto.getTotalPayAmount()).stripTrailingZeros().toPlainString()));

        //计算分摊百分比
        calculatePercentage(headerDetailsDto);

        String platformDiscounts = header.getPlatformDiscounts();
        String merchantDiscounts = header.getMerchantDiscounts();
        String expertDiscounts = header.getExpertDiscounts();
        String payDiscounts = header.getPayDiscounts();

        BigDecimal targetPlatformDiscounts = null;
        BigDecimal targetMerchantDiscounts = null;
        BigDecimal targetExpertDiscounts = null;
        BigDecimal targetPayDiscounts = null;

        //这4个优惠字段，可能没有，也有可能为0！
        if (platformDiscounts != null && !"".equals(platformDiscounts) && !"0".equals(new BigDecimal(platformDiscounts).stripTrailingZeros().toPlainString())) {
            targetPlatformDiscounts = new BigDecimal(platformDiscounts);
        }
        if (merchantDiscounts != null && !"".equals(merchantDiscounts) && !"0".equals(new BigDecimal(merchantDiscounts).stripTrailingZeros().toPlainString())) {
            targetMerchantDiscounts = new BigDecimal(merchantDiscounts);
        }
        if (expertDiscounts != null && !"".equals(expertDiscounts) && !"0".equals(new BigDecimal(expertDiscounts).stripTrailingZeros().toPlainString())) {
            targetExpertDiscounts = new BigDecimal(expertDiscounts);
        }
        if (payDiscounts != null && !"".equals(payDiscounts) && !"0".equals(new BigDecimal(payDiscounts).stripTrailingZeros().toPlainString())) {
            targetPayDiscounts = new BigDecimal(payDiscounts);
        }

        /**
         * OFS销售订单明细行，没有分摊金额的几种情况：
         * 1.对应的OFS销售订单表头某个优惠字段为空或为0
         * 2.明细行没有实付金额或者为0，所以没有分摊比例，不参与分摊
         */
        BigDecimal totalShareTargetPlatformDiscounts = new BigDecimal("0");
        BigDecimal totalShareTargetMerchantDiscounts = new BigDecimal("0");
        BigDecimal totalShareTargetExpertDiscounts = new BigDecimal("0");
        BigDecimal totalShareTargetPayDiscounts = new BigDecimal("0");

        //分摊金额=分摊比例*优惠金额
        for (int i = 0; i < details.size(); i++) {
            DetailsDto detailsDto = details.get(i);
            //分摊比例
            BigDecimal sharingRatio = detailsDto.getSharingRatio();

            //这里应该先8为小数，实际汇总到出库单时，再四舍五入两位小数
            BigDecimal shareTargetPlatformDiscounts = null;
            BigDecimal shareTargetMerchantDiscounts = null;
            BigDecimal shareTargetExpertDiscounts = null;
            BigDecimal shareTargetPayDiscounts = null;
            if (sharingRatio != null && targetPlatformDiscounts != null) {
                shareTargetPlatformDiscounts = targetPlatformDiscounts.multiply(sharingRatio).setScale(8, BigDecimal.ROUND_HALF_UP);
                detailsDto.setOriginalTargetPlatformDiscounts(targetPlatformDiscounts);
                detailsDto.setShareTargetPlatformDiscounts(shareTargetPlatformDiscounts);

                totalShareTargetPlatformDiscounts = totalShareTargetPlatformDiscounts.add(shareTargetPlatformDiscounts);
            }
            if (sharingRatio != null && targetMerchantDiscounts != null) {
                shareTargetMerchantDiscounts = targetMerchantDiscounts.multiply(sharingRatio).setScale(8, BigDecimal.ROUND_HALF_UP);
                detailsDto.setOriginalTargetMerchantDiscounts(targetMerchantDiscounts);
                detailsDto.setShareTargetMerchantDiscounts(shareTargetMerchantDiscounts);

                totalShareTargetMerchantDiscounts = totalShareTargetPlatformDiscounts.add(shareTargetMerchantDiscounts);
            }
            if (sharingRatio != null && targetExpertDiscounts != null) {
                shareTargetExpertDiscounts = targetExpertDiscounts.multiply(sharingRatio).setScale(8, BigDecimal.ROUND_HALF_UP);
                detailsDto.setOriginalTargetExpertDiscounts(targetExpertDiscounts);
                detailsDto.setShareTargetExpertDiscounts(shareTargetExpertDiscounts);

                totalShareTargetExpertDiscounts = totalShareTargetPlatformDiscounts.add(shareTargetExpertDiscounts);
            }
            if (sharingRatio != null && targetPayDiscounts != null) {
                shareTargetPayDiscounts = targetPayDiscounts.multiply(sharingRatio).setScale(8, BigDecimal.ROUND_HALF_UP);
                detailsDto.setOriginalTargetPayDiscounts(targetPayDiscounts);
                detailsDto.setShareTargetPayDiscounts(shareTargetPayDiscounts);

                totalShareTargetPayDiscounts = totalShareTargetPlatformDiscounts.add(shareTargetPayDiscounts);
            }

            //如果是最后一行，则把尾差进行追加
            if (i == details.size() - 1) {
                if (!"0".equals(totalShareTargetPlatformDiscounts.stripTrailingZeros().toPlainString()) && detailsDto.getShareTargetPlatformDiscounts() != null) {
                    //原始优惠金额-sum（分摊金额）
                    BigDecimal subtract = targetPlatformDiscounts.subtract(totalShareTargetPlatformDiscounts);
                    BigDecimal shareTargetPlatformDiscounts1 = detailsDto.getShareTargetPlatformDiscounts();
                    detailsDto.setShareTargetPlatformDiscounts(shareTargetPlatformDiscounts1.add(subtract));
                }
                if (!"0".equals(totalShareTargetMerchantDiscounts.stripTrailingZeros().toPlainString()) && detailsDto.getShareTargetMerchantDiscounts() != null) {
                    //原始优惠金额-sum（分摊金额）
                    BigDecimal subtract = targetMerchantDiscounts.subtract(totalShareTargetMerchantDiscounts);
                    BigDecimal shareTargetPlatformDiscounts1 = detailsDto.getShareTargetPlatformDiscounts();
                    detailsDto.setShareTargetPlatformDiscounts(shareTargetPlatformDiscounts1.add(subtract));
                }
                if (!"0".equals(totalShareTargetExpertDiscounts.stripTrailingZeros().toPlainString()) && detailsDto.getShareTargetExpertDiscounts() != null) {
                    BigDecimal subtract = targetExpertDiscounts.subtract(totalShareTargetExpertDiscounts);
                    BigDecimal shareTargetExpertDiscounts1 = detailsDto.getShareTargetExpertDiscounts();
                    detailsDto.setShareTargetExpertDiscounts(shareTargetExpertDiscounts1.add(subtract));
                }
                if (!"0".equals(totalShareTargetPayDiscounts.stripTrailingZeros().toPlainString()) && detailsDto.getShareTargetPayDiscounts() != null) {
                    BigDecimal subtract = targetPayDiscounts.subtract(totalShareTargetPayDiscounts);
                    BigDecimal shareTargetPayDiscounts1 = detailsDto.getShareTargetPayDiscounts();
                    detailsDto.setShareTargetPayDiscounts(shareTargetPayDiscounts1.add(subtract));
                }
            }

            //拷贝回到原始的集合
            copyDetailsDtoOld(details, detailsOld);
        }
        logger.info("==TOC销售金额分摊结束==");
    }

    /**
     * 计算百分比
     * 分摊比例=实付金额/sum(实付金额)
     *
     * @param headerDetailsDto2 OFS销售订单
     * @author liuyang
     */
    private void calculatePercentage(com.hzya.frame.ttxofs.dto.ofssalesordersearch.HeaderDetailsDto headerDetailsDto2) throws Exception {
        Assert.notNull(headerDetailsDto2, "headerDetailsDto2不能为空");
        HeaderDto header1 = headerDetailsDto2.getHeader();
        List<DetailsDto> details1 = headerDetailsDto2.getDetails();

        BigDecimal sumActualPaymentAmountBigDecimal = sumActualPaymentAmount(details1, header1.getCode());
        for (int i = 0; i < details1.size(); i++) {
            DetailsDto detailsDto = details1.get(i);

            String totalPayAmount = detailsDto.getTotalPayAmount();
            BigDecimal totalPayAmountBigDecimal = new BigDecimal(totalPayAmount);
            //如果实付金额为0则不进行分摊！
            if (!"0".equals(totalPayAmountBigDecimal.stripTrailingZeros().toPlainString())) {
                //得到分摊比例
                BigDecimal percentageBigDecimal = totalPayAmountBigDecimal.divide(sumActualPaymentAmountBigDecimal, 20, BigDecimal.ROUND_HALF_UP).setScale(8, BigDecimal.ROUND_HALF_UP);
                detailsDto.setSharingRatio(percentageBigDecimal);
            }
        }
    }

    /**
     * 累计这一单对应的实付金额
     *
     * @param details1 OFS销售订单明细行
     * @param code     OFS销售订单号
     * @author liuyang
     */
    private BigDecimal sumActualPaymentAmount(List<DetailsDto> details1, String code) throws Exception {
        Assert.notNull(details1, "details1不能为空");
        Assert.notNull(code, "code不能为空");

        BigDecimal actualPaymentAmountBigDecimal = new BigDecimal("0");
        for (int i = 0; i < details1.size(); i++) {
            DetailsDto detailsDto = details1.get(i);

            String totalPayAmount = detailsDto.getTotalPayAmount();
            if (totalPayAmount == null || "".equals(totalPayAmount.trim())) {
                Assert.state(false, "销售订单收付金额不能为空 OFS销售订单号：{}", code);
            }
            BigDecimal totalPayAmountBigDecimal = new BigDecimal(totalPayAmount);
            actualPaymentAmountBigDecimal = actualPaymentAmountBigDecimal.add(totalPayAmountBigDecimal);
        }
        if ("0".equals(actualPaymentAmountBigDecimal.stripTrailingZeros().toPlainString())) {
            Assert.state(false, "销售订单收付金额(合并后)不能为0 OFS销售订单号：{}", code);
        }
        return actualPaymentAmountBigDecimal;
    }

    /**
     * 集合复制
     *
     * @param details 旧OFS销售订单明细行
     * @author liuyang
     */
    private List<DetailsDto> copyDetailsDto(List<DetailsDto> details) throws Exception {
        List<DetailsDto> newDetails = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            DetailsDto detailsDto = details.get(i);

            DetailsDto newDetailsDto1 = new DetailsDto();
            BeanUtil.copyPropertiesV2(detailsDto, newDetailsDto1);
            newDetails.add(newDetailsDto1);
        }
        return newDetails;
    }

    /**
     * 拷贝到原集合中
     *
     * @param newDetails 新OFS销售订单明细行
     * @param detailsOld 旧OFS销售订单明细行
     * @author liuyang
     */
    private void copyDetailsDtoOld(List<DetailsDto> newDetails, List<DetailsDto> detailsOld) throws Exception {
        if (newDetails != null && detailsOld != null) {
            for (int i = 0; i < detailsOld.size(); i++) {
                DetailsDto detailsDto = detailsOld.get(i);
                for (int j = 0; j < newDetails.size(); j++) {
                    DetailsDto detailsDto1 = newDetails.get(j);
                    if (detailsDto.getId().equals(detailsDto1.getId())) {
                        //把分摊后的金额拷贝到旧的OFS销售订单集合中
                        BeanUtil.copyPropertiesV2(detailsDto1, detailsDto);
                    }
                }
            }
        } else {
            logger.error("copyDetailsDtoOld方法newDetails或者detailsOld为空！");
        }
    }
}